//--------------------------------------------------------------
//
//		CALLBACK.DLL
//		Callback Emulator for Centura CTD
//
//		This module maps windows callback procedures to
//		CTD functions
//
//		Author:	Christian Schubert 
//		Email:	christian@cschubert.net
//		Last changed: 2002-03-07
//
//--------------------------------------------------------------

//-------- Includes -----------
//#define _UNICODE
#include <windows.h>
#include <stdarg.h>
#include <tchar.h>
#include "cbtype.h"
#include "centura.h"

#pragma warning( disable : 4996 )

#define BUFFERLENGTH 1000

//-------- Globals ------------
WCHAR	sProcName1[BUFFERLENGTH] = _T("");
WCHAR	sProcName2[BUFFERLENGTH] = _T("");
WCHAR	sProcName3[BUFFERLENGTH] = _T("");
//WCHAR	sCurrentContextValue[BUFFERLENGTH] = _T("");
BOOL	bGlobalReportErrors1;
BOOL	bGlobalReportErrors2;
BOOL	bGlobalReportErrors3;
HANDLE	hGlobalModule;


//-------- ExecuteParamString ------------
//
// Executes the SAL callback function by calling SalCompileAndEvaluate
//
//----------------------------------------
LRESULT ExecuteParamString ( BOOL bReportErrors, LPWSTR sSalString )
{
	int nError, nErrorPos;

	// Dummy result vars
	NUMBER nResult;
	HSTRING sResult;
	DATETIME dResult;
	HWND hResult;
	long nLength, nRet;
	HSTRING sCurrentContext;
	LPTSTR pContext;

	sCurrentContext = SalContextCurrent();
	pContext = SWinHStringLock ( sCurrentContext, &nLength );
	//memset ( sCurrentContextValue, '\0', sizeof ( sCurrentContextValue ) );
	//wcsncpy ( sCurrentContextValue, pContext, min(nLength, BUFFERLENGTH) );

	//if ( 1 == SalCompileAndEvaluate( sSalString, &nError, &nErrorPos, &nResult, &sResult, &dResult, &hResult, bReportErrors, sCurrentContextValue) )
	if ( 1 == SalCompileAndEvaluate( sSalString, &nError, &nErrorPos, &nResult, &sResult, &dResult, &hResult, bReportErrors, pContext) )
		if ( SWinCvtNumberToLong ( &nResult, &nRet ) )
		{
			//SWinHStringUnlock ( sCurrentContext );
			return nRet;
		}


	//SWinHStringUnlock ( sCurrentContext );
	return 0;

}

//-------- TransferParams ------------
//
// Builds a string in SAL syntax and inserts 
// the parameter values as numeric constants.
//
//------------------------------------
LRESULT TransferParams ( LPWSTR sProcName, BOOL bReportErrors, WORD nNumParams, ... )
{
	va_list vl;
	WORD nCount;
	WCHAR sParamNumber[12];
	WCHAR sSalString[254];

	wcscpy ( sSalString, sProcName );

	wcscat ( sSalString, _T("(") );

	// initialize dynamic param list
	va_start( vl, nNumParams );

	for ( nCount=0; nCount<nNumParams; nCount++ )
	{
		// append comma if not first param
		if ( nCount )
			wcscat ( sSalString, _T(",") );
		// convert param to decimal string
		 _itow ( va_arg( vl, int ), sParamNumber, 10 );
		 // append param
		 wcscat ( sSalString, sParamNumber );
	}
		 
	wcscat ( sSalString, _T(")") );

	// end dynamic param processing
	va_end( vl );

	// execute the SAL command
	return ExecuteParamString ( bReportErrors, sSalString );
}


// The following lines contain separate functions for any number of parameters needed

//----1 parameter--------------------------------

LRESULT CALLBACK CallbackProc1_1( DWORD dwParam1 )
{
	return TransferParams ( sProcName1, bGlobalReportErrors1, 1, dwParam1 );
}


//----2 parameters--------------------------------

LRESULT CALLBACK CallbackProc2_1( DWORD dwParam1, DWORD dwParam2 )
{
	return TransferParams ( sProcName1, bGlobalReportErrors1, 2, dwParam1, dwParam2 );
}


//----3 parameters--------------------------------
LRESULT CALLBACK CallbackProc3_1( DWORD dwParam1, DWORD dwParam2, DWORD dwParam3 )
{
	return TransferParams ( sProcName1, bGlobalReportErrors1, 3, dwParam1, dwParam2, dwParam3 );
}


//----4 parameters--------------------------------
LRESULT CALLBACK CallbackProc4_1( DWORD dwParam1, DWORD dwParam2, DWORD dwParam3, DWORD dwParam4)
{
	return TransferParams ( sProcName1, bGlobalReportErrors1, 4, dwParam1, dwParam2, dwParam3, dwParam4 );
}


//----5 parameters--------------------------------
LRESULT CALLBACK CallbackProc5_1( DWORD dwParam1, DWORD dwParam2, DWORD dwParam3, DWORD dwParam4, DWORD dwParam5)
{
	return TransferParams ( sProcName1, bGlobalReportErrors1, 5, dwParam1, dwParam2, dwParam3, dwParam4, dwParam5 );
}


//----6 parameters--------------------------------
LRESULT CALLBACK CallbackProc6_1( DWORD dwParam1, DWORD dwParam2, DWORD dwParam3, DWORD dwParam4, DWORD dwParam5, DWORD dwParam6)
{
	return TransferParams ( sProcName1, bGlobalReportErrors1, 6, dwParam1, dwParam2, dwParam3, dwParam4, dwParam5, dwParam6 );
}
//----1 parameter--------------------------------

LRESULT CALLBACK CallbackProc1_2( DWORD dwParam1 )
{
	return TransferParams ( sProcName2, bGlobalReportErrors2, 1, dwParam1 );
}


//----2 parameters--------------------------------

LRESULT CALLBACK CallbackProc2_2( DWORD dwParam1, DWORD dwParam2 )
{
	return TransferParams ( sProcName2, bGlobalReportErrors2, 2, dwParam1, dwParam2 );
}


//----3 parameters--------------------------------
LRESULT CALLBACK CallbackProc3_2( DWORD dwParam1, DWORD dwParam2, DWORD dwParam3 )
{
	return TransferParams ( sProcName2, bGlobalReportErrors2, 3, dwParam1, dwParam2, dwParam3 );
}


//----4 parameters--------------------------------
LRESULT CALLBACK CallbackProc4_2( DWORD dwParam1, DWORD dwParam2, DWORD dwParam3, DWORD dwParam4)
{
	return TransferParams ( sProcName2, bGlobalReportErrors2, 4, dwParam1, dwParam2, dwParam3, dwParam4 );
}


//----5 parameters--------------------------------
LRESULT CALLBACK CallbackProc5_2( DWORD dwParam1, DWORD dwParam2, DWORD dwParam3, DWORD dwParam4, DWORD dwParam5)
{
	return TransferParams ( sProcName2, bGlobalReportErrors2, 5, dwParam1, dwParam2, dwParam3, dwParam4, dwParam5 );
}


//----6 parameters--------------------------------
LRESULT CALLBACK CallbackProc6_2( DWORD dwParam1, DWORD dwParam2, DWORD dwParam3, DWORD dwParam4, DWORD dwParam5, DWORD dwParam6)
{
	return TransferParams ( sProcName2, bGlobalReportErrors2, 6, dwParam1, dwParam2, dwParam3, dwParam4, dwParam5, dwParam6 );
}
//----1 parameter--------------------------------

LRESULT CALLBACK CallbackProc1_3( DWORD dwParam1 )
{
	return TransferParams ( sProcName3, bGlobalReportErrors3, 1, dwParam1 );
}


//----2 parameters--------------------------------

LRESULT CALLBACK CallbackProc2_3( DWORD dwParam1, DWORD dwParam2 )
{
	return TransferParams ( sProcName3, bGlobalReportErrors3, 2, dwParam1, dwParam2 );
}


//----3 parameters--------------------------------
LRESULT CALLBACK CallbackProc3_3( DWORD dwParam1, DWORD dwParam2, DWORD dwParam3 )
{
	return TransferParams ( sProcName3, bGlobalReportErrors3, 3, dwParam1, dwParam2, dwParam3 );
}


//----4 parameters--------------------------------
LRESULT CALLBACK CallbackProc4_3( DWORD dwParam1, DWORD dwParam2, DWORD dwParam3, DWORD dwParam4)
{
	return TransferParams ( sProcName3, bGlobalReportErrors3, 4, dwParam1, dwParam2, dwParam3, dwParam4 );
}


//----5 parameters--------------------------------
LRESULT CALLBACK CallbackProc5_3( DWORD dwParam1, DWORD dwParam2, DWORD dwParam3, DWORD dwParam4, DWORD dwParam5)
{
	return TransferParams ( sProcName3, bGlobalReportErrors3, 5, dwParam1, dwParam2, dwParam3, dwParam4, dwParam5 );
}


//----6 parameters--------------------------------
LRESULT CALLBACK CallbackProc6_3( DWORD dwParam1, DWORD dwParam2, DWORD dwParam3, DWORD dwParam4, DWORD dwParam5, DWORD dwParam6)
{
	return TransferParams ( sProcName3, bGlobalReportErrors3, 6, dwParam1, dwParam2, dwParam3, dwParam4, dwParam5, dwParam6 );
}


//-----SetCallback------------------------
//
//  Set the name of the SAL callback handler
//  and return the address of the callback 
//  procedure according to the number of 
//  parameters
//
//----------------------------------------

void SetCallback ( LPWSTR sProc, int nNumParams, BOOL bReportErrors, LPVOID* pProc, LPHANDLE pModule )
{
	memset ( sProcName1, '\0', sizeof ( sProcName1 ) );
	wcsncpy ( sProcName1, sProc, BUFFERLENGTH - 1 );
	//wcscpy ( sProcName1, sProc );

	bGlobalReportErrors1 = bReportErrors;

	switch ( nNumParams )
	{
		case 1:
		{
			*pProc = CallbackProc1_1;
			break;
		}
		case 2:
		{
			*pProc = CallbackProc2_1;
			break;
		}
		case 3:
		{
			*pProc = CallbackProc3_1;
			break;
		}
		case 4:
		{
			*pProc = CallbackProc4_1;
			break;
		}
		case 5:
		{
			*pProc = CallbackProc5_1;
			break;
		}
		case 6:
		{
			*pProc = CallbackProc6_1;
			break;
		}
		default:
		{
			*pProc = NULL;
		}
	}

	*pModule = hGlobalModule;
}
void SetCallback2 ( LPWSTR sProc, int nNumParams, BOOL bReportErrors, LPVOID* pProc, LPHANDLE pModule )
{
	memset ( (LPVOID)sProcName2, '\0', sizeof ( sProcName2 ) );
	wcsncpy ( sProcName2, sProc, BUFFERLENGTH - 1 );

	bGlobalReportErrors2 = bReportErrors;

	switch ( nNumParams )
	{
		case 1:
		{
			*pProc = CallbackProc1_2;
			break;
		}
		case 2:
		{
			*pProc = CallbackProc2_2;
			break;
		}
		case 3:
		{
			*pProc = CallbackProc3_2;
			break;
		}
		case 4:
		{
			*pProc = CallbackProc4_2;
			break;
		}
		case 5:
		{
			*pProc = CallbackProc5_2;
			break;
		}
		case 6:
		{
			*pProc = CallbackProc6_2;
			break;
		}
		default:
		{
			*pProc = NULL;
		}
	}

	*pModule = hGlobalModule;
}
void SetCallback3 ( LPWSTR sProc, int nNumParams, BOOL bReportErrors, LPVOID* pProc, LPHANDLE pModule )
{
	memset ( sProcName3, '\0', sizeof ( sProcName3 ) );
	wcsncpy ( sProcName3, sProc, BUFFERLENGTH - 1 );

	bGlobalReportErrors3 = bReportErrors;

	switch ( nNumParams )
	{
		case 1:
		{
			*pProc = CallbackProc1_3;
			break;
		}
		case 2:
		{
			*pProc = CallbackProc2_3;
			break;
		}
		case 3:
		{
			*pProc = CallbackProc3_3;
			break;
		}
		case 4:
		{
			*pProc = CallbackProc4_3;
			break;
		}
		case 5:
		{
			*pProc = CallbackProc5_3;
			break;
		}
		case 6:
		{
			*pProc = CallbackProc6_3;
			break;
		}
		default:
		{
			*pProc = NULL;
		}
	}

	*pModule = hGlobalModule;
}


//-----DllMain------------------------
BOOL APIENTRY DllMain( HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved )
{
	hGlobalModule = hModule;

    return 1;
}


